from sklearn_compat._sklearn_compat import (
    is_df_or_series,  # noqa: F401
    is_pandas_df_or_series,  # noqa: F401
    is_pandas_df,  # noqa: F401
    is_polars_df_or_series,  # noqa: F401
    is_polars_df,  # noqa: F401
    is_pyarrow_data,  # noqa: F401
)


"""Tests for dataframe detection functions."""

import numpy as np
import pytest

from sklearn._min_dependencies import dependent_packages
from sklearn.utils._testing import _convert_container

from sklearn_compat._sklearn_compat import parse_version, sklearn_version
from sklearn_compat.utils._dataframe import (
    is_df_or_series,
    is_pandas_df,
    is_pandas_df_or_series,
    is_polars_df,
    is_polars_df_or_series,
    is_pyarrow_data,
)


@pytest.mark.parametrize(
    "constructor_name",
    [
        pytest.param(
            "pyarrow",
            marks=pytest.mark.skipif(
                "pyarrow" not in dependent_packages,
                reason="pyarrow not in dependent_packages",
            ),
        ),
        "dataframe",
        pytest.param(
            "polars",
            marks=pytest.mark.skipif(
                "polars" not in dependent_packages,
                reason="polars not in dependent_packages",
            ),
        ),
    ],
)
def test_is_df_or_series(constructor_name):
    df = _convert_container([[1, 4, 2], [3, 3, 6]], constructor_name)

    assert is_df_or_series(df)
    assert not is_df_or_series(np.asarray([1, 2, 3]))


@pytest.mark.parametrize(
    "constructor_name",
    [
        pytest.param(
            "pyarrow",
            marks=pytest.mark.skipif(
                "pyarrow" not in dependent_packages,
                reason="pyarrow not in dependent_packages",
            ),
        ),
        "dataframe",
        pytest.param(
            "polars",
            marks=pytest.mark.skipif(
                "polars" not in dependent_packages,
                reason="polars not in dependent_packages",
            ),
        ),
    ],
)
def test_is_pandas_df_other_libraries(constructor_name):
    df = _convert_container([[1, 4, 2], [3, 3, 6]], constructor_name)
    if constructor_name in ("pyarrow", "polars"):
        assert not is_pandas_df(df)
    else:
        assert is_pandas_df(df)


def test_is_pandas_df():
    """Check behavior of is_pandas_df when pandas is installed."""
    pd = pytest.importorskip("pandas")
    df = pd.DataFrame([[1, 2, 3]])
    assert is_pandas_df(df)
    assert not is_pandas_df(np.asarray([1, 2, 3]))
    assert not is_pandas_df(1)


def test_is_pandas_df_pandas_not_installed(hide_available_pandas):
    """Check is_pandas_df when pandas is not installed."""

    assert not is_pandas_df(np.asarray([1, 2, 3]))
    assert not is_pandas_df(1)


@pytest.mark.parametrize(
    "constructor_name, minversion",
    [
        pytest.param(
            "pyarrow",
            dependent_packages.get("pyarrow", [None])[0],
            marks=pytest.mark.skipif(
                "pyarrow" not in dependent_packages
                or sklearn_version < parse_version("1.4"),
                reason="pyarrow not in dependent_packages or sklearn < 1.4",
            ),
        ),
        ("dataframe", dependent_packages["pandas"][0]),
        pytest.param(
            "polars",
            dependent_packages.get("polars", [None])[0],
            marks=pytest.mark.skipif(
                "polars" not in dependent_packages
                or sklearn_version < parse_version("1.4"),
                reason="polars not in dependent_packages or sklearn < 1.4",
            ),
        ),
    ],
)
def test_is_polars_df_other_libraries(constructor_name, minversion):
    if constructor_name == "dataframe":
        df = _convert_container([[1, 4, 2], [3, 3, 6]], constructor_name)
    else:
        df = _convert_container(
            [[1, 4, 2], [3, 3, 6]],
            constructor_name,
            minversion=minversion,
        )
    if constructor_name in ("pyarrow", "dataframe"):
        assert not is_polars_df(df)
    else:
        assert is_polars_df(df)


def test_is_polars_df_for_duck_typed_polars_dataframe():
    """Check is_polars_df for object that looks like a polars dataframe"""

    class NotAPolarsDataFrame:
        def __init__(self):
            self.columns = [1, 2, 3]
            self.schema = "my_schema"

    not_a_polars_df = NotAPolarsDataFrame()
    assert not is_polars_df(not_a_polars_df)


def test_is_polars_df():
    """Check that is_polars_df return False for non-dataframe objects."""

    class LooksLikePolars:
        def __init__(self):
            self.columns = ["a", "b"]
            self.schema = ["a", "b"]

    assert not is_polars_df(LooksLikePolars())


def test_is_pandas_df_or_series():
    """Check behavior of is_pandas_df_or_series when pandas is installed."""
    pd = pytest.importorskip("pandas")
    df = pd.DataFrame([[1, 2, 3]])
    series = pd.Series([1, 2, 3])
    assert is_pandas_df_or_series(df)
    assert is_pandas_df_or_series(series)
    assert not is_pandas_df_or_series(np.asarray([1, 2, 3]))
    assert not is_pandas_df_or_series(1)


def test_is_pandas_df_or_series_pandas_not_installed(hide_available_pandas):
    """Check is_pandas_df_or_series when pandas is not installed."""
    assert not is_pandas_df_or_series(np.asarray([1, 2, 3]))
    assert not is_pandas_df_or_series(1)


@pytest.mark.parametrize(
    "constructor_name",
    [
        pytest.param(
            "pyarrow",
            marks=pytest.mark.skipif(
                "pyarrow" not in dependent_packages,
                reason="pyarrow not in dependent_packages",
            ),
        ),
        "dataframe",
        pytest.param(
            "polars",
            marks=pytest.mark.skipif(
                "polars" not in dependent_packages,
                reason="polars not in dependent_packages",
            ),
        ),
    ],
)
def test_is_pandas_df_or_series_other_libraries(constructor_name):
    df = _convert_container([[1, 4, 2], [3, 3, 6]], constructor_name)
    if constructor_name in ("pyarrow", "polars"):
        assert not is_pandas_df_or_series(df)
    else:
        assert is_pandas_df_or_series(df)


def test_is_polars_df_or_series():
    """Check behavior of is_polars_df_or_series when polars is installed."""
    pl = pytest.importorskip("polars")
    df = pl.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
    series = pl.Series("a", [1, 2, 3])
    assert is_polars_df_or_series(df)
    assert is_polars_df_or_series(series)
    assert not is_polars_df_or_series(np.asarray([1, 2, 3]))
    assert not is_polars_df_or_series(1)


@pytest.mark.parametrize(
    "constructor_name, minversion",
    [
        pytest.param(
            "pyarrow",
            dependent_packages.get("pyarrow", [None])[0],
            marks=pytest.mark.skipif(
                "pyarrow" not in dependent_packages
                or sklearn_version < parse_version("1.4"),
                reason="pyarrow not in dependent_packages or sklearn < 1.4",
            ),
        ),
        ("dataframe", dependent_packages["pandas"][0]),
        pytest.param(
            "polars",
            dependent_packages.get("polars", [None])[0],
            marks=pytest.mark.skipif(
                "polars" not in dependent_packages
                or sklearn_version < parse_version("1.4"),
                reason="polars not in dependent_packages or sklearn < 1.4",
            ),
        ),
    ],
)
def test_is_polars_df_or_series_other_libraries(constructor_name, minversion):
    if constructor_name == "dataframe":
        df = _convert_container([[1, 4, 2], [3, 3, 6]], constructor_name)
    else:
        df = _convert_container(
            [[1, 4, 2], [3, 3, 6]],
            constructor_name,
            minversion=minversion,
        )
    if constructor_name in ("pyarrow", "dataframe"):
        assert not is_polars_df_or_series(df)
    else:
        assert is_polars_df_or_series(df)


def test_is_pyarrow_data():
    """Check behavior of is_pyarrow_data when pyarrow is installed."""
    pa = pytest.importorskip("pyarrow")
    table = pa.table({"a": [1, 2, 3], "b": [4, 5, 6]})
    record_batch = pa.record_batch({"a": [1, 2, 3], "b": [4, 5, 6]})
    array = pa.array([1, 2, 3])
    chunked_array = pa.chunked_array([[1, 2, 3], [4, 5, 6]])
    assert is_pyarrow_data(table)
    assert is_pyarrow_data(record_batch)
    assert is_pyarrow_data(array)
    assert is_pyarrow_data(chunked_array)
    assert not is_pyarrow_data(np.asarray([1, 2, 3]))
    assert not is_pyarrow_data(1)


@pytest.mark.parametrize(
    "constructor_name, minversion",
    [
        pytest.param(
            "pyarrow",
            dependent_packages.get("pyarrow", [None])[0],
            marks=pytest.mark.skipif(
                "pyarrow" not in dependent_packages
                or sklearn_version < parse_version("1.4"),
                reason="pyarrow not in dependent_packages or sklearn < 1.4",
            ),
        ),
        ("dataframe", dependent_packages["pandas"][0]),
        pytest.param(
            "polars",
            dependent_packages.get("polars", [None])[0],
            marks=pytest.mark.skipif(
                "polars" not in dependent_packages
                or sklearn_version < parse_version("1.4"),
                reason="polars not in dependent_packages or sklearn < 1.4",
            ),
        ),
    ],
)
def test_is_pyarrow_data_other_libraries(constructor_name, minversion):
    if constructor_name == "dataframe":
        df = _convert_container([[1, 4, 2], [3, 3, 6]], constructor_name)
    else:
        df = _convert_container(
            [[1, 4, 2], [3, 3, 6]],
            constructor_name,
            minversion=minversion,
        )
    if constructor_name == "pyarrow":
        assert is_pyarrow_data(df)
    else:
        assert not is_pyarrow_data(df)



import numpy as np
import pytest
from sklearn import config_context
from sklearn.base import BaseEstimator, ClassifierMixin, MetaEstimatorMixin, clone

from sklearn_compat._sklearn_compat import parse_version, sklearn_version
from sklearn_compat.utils.metadata_routing import _raise_for_params, process_routing


@pytest.mark.skipif(
    sklearn_version < parse_version("1.3"),
    reason="Metadata routing was introduced in scikit-learn 1.3",
)
def test_process_routing():
    from sklearn.utils.metadata_routing import MetadataRouter, MethodMapping

    class ExampleClassifier(ClassifierMixin, BaseEstimator):
        def fit(self, X, y, sample_weight=None):
            self.sample_weight_ = sample_weight
            # all classifiers need to expose a classes_ attribute once they're fit.
            self.classes_ = np.array([0, 1])
            return self

    class MetaClassifier(MetaEstimatorMixin, ClassifierMixin, BaseEstimator):
        def __init__(self, estimator):
            self.estimator = estimator

        def fit(self, X, y, sample_weight=None):
            routed_params = process_routing(self, "fit", sample_weight=sample_weight)
            self.estimator_ = clone(self.estimator).fit(
                X, y, **routed_params.estimator.fit
            )
            return self

        def get_metadata_routing(self):
            router = MetadataRouter(owner=self.__class__.__name__).add(
                estimator=self.estimator,
                method_mapping=MethodMapping().add(caller="fit", callee="fit"),
            )
            return router

    with config_context(enable_metadata_routing=True):
        est = MetaClassifier(ExampleClassifier().set_fit_request(sample_weight=True))
        sample_weight = [1, 2, 3]
        est.fit(None, None, sample_weight=sample_weight)
        assert est.estimator_.sample_weight_ == sample_weight


@pytest.mark.skipif(
    sklearn_version < parse_version("1.3"),
    reason="Metadata routing was introduced in scikit-learn 1.3",
)
def test_raise_for_params():
    class DummyEstimator(BaseEstimator):
        pass

    est = DummyEstimator()
    params = {"invalid_param": 42}

    with pytest.raises(ValueError, match="Passing extra keyword arguments"):
        _raise_for_params(params, est, "fit")


@pytest.mark.skipif(
    sklearn_version >= parse_version("1.3"),
    reason="Metadata routing is implemented",
)
def test__raise_for_params_not_implemented():
    with pytest.raises(NotImplementedError):
        _raise_for_params(None, None, "fit")
    with pytest.raises(NotImplementedError):
        process_routing(None, "fit")



import pytest
from sklearn.base import BaseEstimator

from sklearn_compat._sklearn_compat import parse_version, sklearn_version
from sklearn_compat.utils._tags import get_tags


def test_get_tags():
    class MyEstimator(BaseEstimator):
        def _more_tags(self):
            return {"requires_fit": False}

        def __sklearn_tags__(self):
            tags = super().__sklearn_tags__()
            tags.requires_fit = False
            return tags

    tags = get_tags(MyEstimator())
    assert not tags.requires_fit


@pytest.mark.skipif(
    sklearn_version >= parse_version("1.6"),
    reason="Test only relevant for sklearn < 1.6",
)
def test_patched_more_tags():
    """Non-regression test for:
    https://github.com/sklearn-compat/sklearn-compat/issues/27

    The regression can be spotted when an estimator dynamically updates the tags
    based on the inner estimator.
    """

    class MyEstimator(BaseEstimator):
        def __init__(self, allow_nan=False):
            self.allow_nan = allow_nan

        def _more_tags(self):
            return {"allow_nan": self.allow_nan}

        def __sklearn_tags__(self):
            tags = super().__sklearn_tags__()
            tags.input_tags.allow_nan = self.allow_nan
            return tags

    class MetaEstimator(BaseEstimator):
        def __init__(self, estimator):
            self.estimator = estimator

        def fit(self, X, y=None):
            return self

        def _more_tags(self):
            return {
                "allow_nan": get_tags(self.estimator).input_tags.allow_nan,
            }

        def __sklearn_tags__(self):
            tags = super().__sklearn_tags__()
            tags.input_tags.allow_nan = get_tags(self.estimator).input_tags.allow_nan
            return tags

    est_no_support_nan = MetaEstimator(estimator=MyEstimator(allow_nan=False))
    est_support_nan = MetaEstimator(estimator=MyEstimator(allow_nan=True))

    assert not get_tags(est_no_support_nan).input_tags.allow_nan
    assert get_tags(est_support_nan).input_tags.allow_nan

    from sklearn_compat._sklearn_compat import _patched_more_tags

    test_to_fail = [{"some_check": True}]
    _patched_more_tags(est_no_support_nan, expected_failed_checks=test_to_fail)
    _patched_more_tags(est_support_nan, expected_failed_checks=test_to_fail)

    # check that patching the instance to add the test to fail should not overwrite
    # other tags
    assert not get_tags(est_no_support_nan).input_tags.allow_nan
    assert get_tags(est_support_nan).input_tags.allow_nan
    # check that accessing the _xfail_checks via the old _get_tags API report the
    # test to be skipped
    assert est_no_support_nan._get_tags()["_xfail_checks"] == test_to_fail
    assert est_support_nan._get_tags()["_xfail_checks"] == test_to_fail



import pytest
from sklearn.utils._param_validation import InvalidParameterError

from sklearn_compat.utils._param_validation import validate_params


def test_validate_params():
    @validate_params({"x": [int, float]}, prefer_skip_nested_validation=True)
    def func(x):
        return x

    func(1)
    func(1.0)
    with pytest.raises(InvalidParameterError):
        func("a")



from sklearn_compat.utils.fixes import (
    _IS_32BIT,
    _IS_WASM,
    _in_unstable_openblas_configuration,
)


def test__in_unstable_openblas_configuration():
    _in_unstable_openblas_configuration()


def test__IS_WASM():
    assert not _IS_WASM


def test__IS_32BIT():
    assert not _IS_32BIT



from sklearn_compat._sklearn_compat import (
    ClassifierTags,  # noqa: F401
    InputTags,  # noqa: F401
    RegressorTags,  # noqa: F401
    Tags,  # noqa: F401
    TargetTags,  # noqa: F401
    TransformerTags,  # noqa: F401
    get_tags,  # noqa: F401
)



from sklearn.base import BaseEstimator, TransformerMixin

from sklearn_compat.utils.estimator_checks import (
    check_estimator,
    parametrize_with_checks,
)
from sklearn_compat.utils.validation import validate_data


class MyEstimator(TransformerMixin, BaseEstimator):
    def fit(self, X, y=None):
        validate_data(self, X=X, y=y, reset=True)
        return self

    def transform(self, X):
        validate_data(self, X=X, reset=False)
        return X


failing_checks = {
    "check_transformer_data_not_an_array": "test",
    "check_transformers_unfitted": "test",
}


def test_check_estimator():
    check_estimator(
        MyEstimator(),
        expected_failed_checks=failing_checks,
    )


@parametrize_with_checks(
    [MyEstimator()], expected_failed_checks=lambda x: failing_checks
)
def test_parametrize_with_checks(estimator, check):
    check(estimator)



import numpy as np
import pytest
from sklearn.base import (
    BaseEstimator,
    ClassifierMixin,
    RegressorMixin,
    TransformerMixin,
)
from sklearn.datasets import make_classification
from sklearn.utils._param_validation import Integral, Interval, StrOptions
from sklearn.utils.estimator_checks import parametrize_with_checks
from sklearn.utils.fixes import parse_version
from sklearn.utils.multiclass import check_classification_targets
from sklearn.utils.validation import check_is_fitted

from sklearn_compat._sklearn_compat import sklearn_version
from sklearn_compat.base import _fit_context
from sklearn_compat.utils.validation import validate_data


class Classifier(ClassifierMixin, BaseEstimator):
    _parameter_constraints = {
        "seed": [Interval(Integral, 0, None, closed="left")],
    }
    _estimator_type = "classifier"

    def __init__(self, seed=0):
        self.seed = seed

    @_fit_context(prefer_skip_nested_validation=True)
    def fit(self, X, y):
        X, y = validate_data(self, X=X, y=y)
        check_classification_targets(y)
        self.classes_ = np.unique(y)
        self.rng_ = np.random.default_rng(self.seed)
        return self

    def predict(self, X):
        check_is_fitted(self)
        X = validate_data(self, X=X, reset=False)
        indices = self.rng_.choice(len(self.classes_), size=X.shape[0], replace=True)
        return self.classes_[indices]

    def _more_tags(self):
        return {"non_deterministic": True, "poor_score": True}

    def __sklearn_tags__(self):
        tags = super().__sklearn_tags__()
        tags.non_deterministic = True
        tags.classifier_tags.poor_score = True
        return tags


class Regressor(RegressorMixin, BaseEstimator):
    _parameter_constraints = {
        "seed": [Interval(Integral, 0, None, closed="left")],
    }
    _estimator_type = "regressor"

    def __init__(self, seed=0):
        self.seed = seed

    @_fit_context(prefer_skip_nested_validation=True)
    def fit(self, X, y):
        X, y = validate_data(self, X=X, y=y)
        self._min, self._max = y.min(), y.max()
        self.rng_ = np.random.default_rng(self.seed)
        return self

    def predict(self, X):
        check_is_fitted(self)
        X = validate_data(self, X=X, reset=False)
        return self.rng_.uniform(self._min, self._max, size=X.shape[0])

    def _more_tags(self):
        return {"non_deterministic": True, "poor_score": True}

    def __sklearn_tags__(self):
        tags = super().__sklearn_tags__()
        tags.non_deterministic = True
        tags.regressor_tags.poor_score = True
        return tags


class Transformer(TransformerMixin, BaseEstimator):
    _parameter_constraints = {"with_mean": ["boolean"], "with_std": ["boolean"]}
    _estimator_type = "transformer"

    def __init__(self, with_mean=True, with_std=True):
        self.with_mean = with_mean
        self.with_std = with_std

    @_fit_context(prefer_skip_nested_validation=True)
    def fit(self, X, y=None):
        X = validate_data(self, X=X)
        self.mean_ = X.mean() if self.with_mean else 0
        self.std_ = X.std() if self.with_std else 1
        return self

    def transform(self, X):
        X = validate_data(self, X=X, reset=False)
        return (X - self.mean_) / self.std_


@parametrize_with_checks(
    [
        Classifier(),
        Regressor(),
        Transformer(),
    ]
)
def test_basic_estimator(estimator, check):
    return check(estimator)



from sklearn_compat._sklearn_compat import (
    _determine_key_type,  # noqa: F401
    _get_column_indices,  # noqa: F401
    _safe_assign,  # noqa: F401
    _safe_indexing,  # noqa: F401
    resample,  # noqa: F401
    shuffle,  # noqa: F401
)



from sklearn_compat.metrics._classification import _check_targets


def test__check_targets_returns_four_values():
    """Check that `_check_targets` always returns 4 values.

    This ensures compatibility across different sklearn versions by guaranteeing
    that the function always returns (y_type, y_true, y_pred, sample_weight).
    """
    y_true = [0, 1, 0, 1]
    y_pred = [0, 1, 0, 1]

    # Test without sample_weight
    result = _check_targets(y_true, y_pred)
    assert len(result) == 4
    y_type, y_true_out, y_pred_out, sample_weight_out = result
    assert sample_weight_out is None

    # Test with sample_weight
    sample_weight = [1.0, 2.0, 1.0, 2.0]
    result = _check_targets(y_true, y_pred, sample_weight=sample_weight)
    assert len(result) == 4
    y_type, y_true_out, y_pred_out, sample_weight_out = result
    assert sample_weight_out is not None



from sklearn.cluster import KMeans
from sklearn.linear_model import LinearRegression

from sklearn_compat.base import is_clusterer


def test_is_clusterer():
    assert is_clusterer(KMeans())
    assert not is_clusterer(LinearRegression())



import pytest

from sklearn_compat.utils._optional_dependencies import (
    check_matplotlib_support,
    check_pandas_support,
)


def test_check_matplotlib_support():
    is_matplotlib_installed = False
    try:
        import matplotlib  # noqa: F401

        is_matplotlib_installed = True
    except ImportError:
        pass

    if is_matplotlib_installed:
        check_matplotlib_support("sklearn_compat")
    else:
        with pytest.raises(ImportError):
            check_matplotlib_support("sklearn_compat")


def test_check_pandas_support():
    is_pandas_installed = False
    try:
        import pandas  # noqa: F401

        is_pandas_installed = True
    except ImportError:
        pass

    if is_pandas_installed:
        check_pandas_support("sklearn_compat")
    else:
        with pytest.raises(ImportError):
            check_pandas_support("sklearn_compat")



from sklearn.linear_model import LinearRegression

from sklearn_compat.utils._test_common.instance_generator import _construct_instances


def test__construct_instances():
    list(iter(_construct_instances(LinearRegression)))



import numpy as np
import pytest
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.utils._testing import _convert_container

from sklearn_compat.utils.validation import (
    _check_feature_names,
    _check_n_features,
    _is_fitted,
    _to_object_array,
    check_array,
    check_X_y,
    validate_data,
)


def test_check_array_ensure_all_finite():
    X = [[1, 2, 3, np.nan]]
    with pytest.raises(ValueError, match="contains NaN"):
        check_array(X, ensure_all_finite=True)
    assert isinstance(check_array(X, ensure_all_finite=False), np.ndarray)


def test_check_X_y_ensure_all_finite():
    X, y = [[1, 2, 3, np.nan]], [1]
    with pytest.raises(ValueError, match="contains NaN"):
        check_X_y(X, y, ensure_all_finite=True)
    X_, y_ = check_X_y(X, y, ensure_all_finite=False)
    assert isinstance(X_, np.ndarray) and isinstance(y_, np.ndarray)


@pytest.mark.parametrize("ensure_all_finite", [True, False])
def test_validate_data(ensure_all_finite):
    """Check the behaviour of `validate_data`.

    This change has been introduced in scikit-learn 1.6.
    """

    class MyEstimator(TransformerMixin, BaseEstimator):
        def fit(self, X, y=None):
            X = validate_data(self, X=X, y=y, ensure_all_finite=ensure_all_finite)
            return self

        def transform(self, X):
            X = validate_data(self, X=X, ensure_all_finite=ensure_all_finite)
            return X

    X = [[1, 2, 3, 4]]
    est = MyEstimator()
    assert isinstance(est.fit_transform(X), np.ndarray)

    X = [[1, 2, 3, np.nan]]
    est = MyEstimator()
    if ensure_all_finite:
        with pytest.raises(ValueError, match="contains NaN"):
            est.fit_transform(X)
    else:
        assert isinstance(est.fit_transform(X), np.ndarray)


@pytest.mark.parametrize("container_type", ["list", "dataframe"])
def test_validate_data_skip_check_array(container_type):
    class MyEstimator(TransformerMixin, BaseEstimator):
        def fit(self, X, y=None):
            X = validate_data(self, X=X, y=y, skip_check_array=True)
            return self

        def transform(self, X):
            X = validate_data(self, X=X, skip_check_array=True)
            return X

    X = _convert_container(
        [[1, 2, 3, 4]], container_type, columns_name=["a", "b", "c", "d"]
    )
    est = MyEstimator()
    X_trans = est.fit_transform(X)
    assert isinstance(X_trans, type(X))
    assert est.n_features_in_ == 4
    if container_type == "dataframe":
        np.testing.assert_array_equal(est.feature_names_in_, ["a", "b", "c", "d"])


def test_check_n_features():
    """Check the behaviour of `_check_n_features`.

    This change has been introduced in scikit-learn 1.6.
    """

    class MyEstimator(TransformerMixin, BaseEstimator):
        def fit(self, X, y=None):
            X = _check_n_features(self, X, reset=True)
            return self

    X = [[1, 2, 3, 4]]
    est = MyEstimator().fit(X)
    assert est.n_features_in_ == 4


def test_check_feature_names():
    """Check the behaviour of `_check_feature_names`.

    This change has been introduced in scikit-learn 1.6.
    """
    pd = pytest.importorskip("pandas")

    class MyEstimator(TransformerMixin, BaseEstimator):
        def fit(self, X, y=None):
            X = _check_feature_names(self, X, reset=True)
            return self

    X = pd.DataFrame([[1, 2, 3, 4]], columns=["a", "b", "c", "d"])
    est = MyEstimator().fit(X)
    np.testing.assert_array_equal(est.feature_names_in_, ["a", "b", "c", "d"])


def test__to_object_array():
    result = _to_object_array([np.array([0]), np.array([1])])
    assert isinstance(result, np.ndarray)
    assert result.dtype == object


def test__is_fitted():
    estimator = BaseEstimator()
    assert not _is_fitted(estimator)




from sklearn_compat.utils._tags import (
    ClassifierTags,
    InputTags,
    RegressorTags,
    Tags,
    TargetTags,
    TransformerTags,
    get_tags,
)

__all__ = [
    "Tags",
    "InputTags",
    "TargetTags",
    "ClassifierTags",
    "RegressorTags",
    "TransformerTags",
    "get_tags",
]
